<?php

namespace UBRIR;

include_once 'WordpressUBRiRConnector.php';

include_once 'ubrr.core/_autoload.php';

require_once($_SERVER['DOCUMENT_ROOT'] . '/wp-load.php');

class Wordpress extends GatewayInteractionPanel
{
    static $known_shops_plugins = Array(
        'WP eCommerce' => Array(
            'min_version' => '3.12.0',
            'file' => 'ecom_ubrr_merchant.php',
            'folder_name' => 'wp-e-commerce/wpsc-merchants/',
            'settings_link' => '/wp-admin/options-general.php?page=wpsc-settings&tab=gateway&payment_gateway_id=wpsc_merchant_ubrr',
            'shop_settings_file' => 'views/shop_settings/ecom.php',
            'settings_shopcode' => 'ecom'
        ),
        'WooCommerce' => Array(
            'min_version' => '2.6.14',
            'file' => 'class-wc-gateway-ubrr.php',
            'folder_name' => 'woocommerce/includes/gateways/ubrr/',
            'settings_link' => '/wp-admin/admin.php?page=wc-settings&tab=checkout&section=ubrr',
            'shop_settings_file' => 'views/shop_settings/woo.php',
            'settings_shopcode' => 'woo'
        ),
        'WP-Shop' => Array(
            'min_version' => '3.8.2',
            'file' => 'wpshop_merchant.php',
            'folder_name' => 'wp-shop-original/',
            'settings_link' => '321',
            'shop_settings_file' => 'views/shop_settings/wpshop.php',
            'settings_shopcode' => 'wpshop'
        ),
    );

    static $optionList = Array(
        'shop_id' =>
            Array(
                'GROUP' => 'UBRR_SETTINGS',
                'title' => 'Номер ретейлера',
                'type' => 'text',
                'value' => ''
            ),
        'certificate_password' =>
            Array(
                'GROUP' => 'UBRR_SETTINGS',
                'title' => 'Пароль к сертификату',
                'type' => 'text',
                'value' => ''
            ),
        'test_mode' =>
            Array(
                'GROUP' => 'UBRR_SETTINGS',
                'title' => 'Тестовый режим',
                'type' => 'checkbox',
                'value' => ''
            ),
        'post_payment' =>
            Array(
                'GROUP' => 'UBRR_SETTINGS',
                'title' => 'Отложенный платеж',
                'type' => 'checkbox',
                'value' => '',
                'help' =>
                    '<span>Отложенный платеж</span> - крайне загадочная вещь'
            ),
        'ubrr_shop_name' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS_SHOP_NAME',
                'title' => 'Название магазина',
                'type' => 'text',
                'value' => ''
            ),
        'ubrr_shop_name_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS_SHOP_NAME',
                'title' => 'Указывать название магазина (Д\Н)',
                'type' => 'checkbox',
                'value' => ''
            ),

        'ubrr_email' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS_EMAIL',
                'title' => 'Email администратора магазина (для платежных уведомлений)',
                'type' => 'text',
                'value' => ''
            ),
        'ubrr_email_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS_EMAIL',
                'title' => 'Отправлять почту (Д\Н)',
                'type' => 'checkbox',
                'value' => ''
            ),

        'ubrr_notifications_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Включить уведомления по e-mail',
                'type' => 'checkbox',
                'value' => ''
            ),

        'ubrr_shop_site_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Ссылка на сайт магазина',
                'type' => 'checkbox',
                'value' => ''
            ),

        'ubrr_shop_email' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'E-mail магазина',
                'type' => 'checkbox',
                'value' => ''
            ),
        'ubrr_amount_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Сумма операции',
                'type' => 'checkbox',
                'value' => ''
            ),
        'ubrr_date_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Дата операции',
                'type' => 'checkbox',
                'value' => ''
            ),
        'ubrr_ordernum_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Номер заказа',
                'type' => 'checkbox',
                'value' => ''
            ),
        'ubrr_cardnum_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Номер карты',
                'type' => 'checkbox',
                'value' => ''
            ),
        'ubrr_cardholder_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Держатель карты',
                'type' => 'checkbox',
                'value' => ''
            ),
        'ubrr_opertype_y' =>
            Array(
                'GROUP' => 'UBRR_NOTIFICATIONS',
                'title' => 'Тип операции',
                'type' => 'checkbox',
                'value' => ''
            ),
    );

    function __construct($useLogs = true)
    {
        //Добавляем настройки статусов в зависимости от плагина
        self::ExtendSettings();

        $wordpressConnector = new \WordpressUBRiRConnector();
        parent::__construct($wordpressConnector, $useLogs);
    }

    public function __destruct()
    {
        parent::__destruct(); // TODO: Change the autogenerated stub
    }

    public function PluginUninstall()
    {
        foreach (self::$known_shops_plugins as $plugin) {
            unlink($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/' . $plugin['folder_name'] . $plugin['file']);
        }
    }

    public function UpdateSettings()
    {
        //Обновляем настройки если форма отправлена
        if (isset($_POST['update_settings_btn'])) {
            // "Защита" от хакеров
            if (function_exists('current_user_can') && !current_user_can('manage_options')) {
                die (_e('Нет доступа', 'ubrr'));
            }
            if (function_exists('check_admin_referer')) {
                check_admin_referer('update_settings');
            }

            foreach (self::$optionList as $key => $value) {
                if ($value['GROUP'] == 'UBRR_SETTINGS') {
                    update_option($key, $_POST[$key]);
                }
            }
        }
    }

    function UpdateShopSettings()
    {
        if (isset($_POST['ubrr_update_shop_settings_btn'])) {

            //Записываем значения полей

            foreach (self::$optionList as $key => $value) {
                if (isset($_POST[$key])) {
                    update_option($key, $_POST[$key]);
                }
            }
        }
    }

    function UpdateNotificationsSettings()
    {
        //Обновляем настройки если форма отправлена
        if (isset($_POST['update_notifications_settings_btn'])) {
            // "Защита" от хакеров
            if (function_exists('current_user_can') && !current_user_can('manage_options')) {
                die (_e('Нет доступа', 'ubrr'));
            }
            if (function_exists('check_admin_referer')) {
                check_admin_referer('update_notifications_settings');
            }
            //Записываем значения полей

            foreach (self::$optionList as $key => $value) {
                if ($value['GROUP'] == 'UBRR_NOTIFICATIONS') {
                    update_option($key, $_POST[$key]);
                }
            }
        }
    }

    static function ExtendSettings()
    {
        foreach (self::$known_shops_plugins as $plugin) {
            $shopcode = $plugin['settings_shopcode'];
            $init_key = 'ubrr_payment_setting_init_' . $shopcode;
            $approved_key = 'ubrr_payment_setting_approved_' . $shopcode;
            $declined_key = 'ubrr_payment_setting_decline_' . $shopcode;
            $canceled_key = 'ubrr_payment_setting_canceled_' . $shopcode;
            $group = 'SHOP_SETTINGS_' . strtoupper($shopcode);
            $ubrr_payment_init =
                Array(
                    'GROUP' => $group,
                    'title' => 'Статус заказа после инциирования оплаты',
                    'type' => 'select',
                    'value' => '',
                );
            $ubrr_payment_approved =
                Array(
                    'GROUP' => $group,
                    'title' => 'Статус заказа после успешной оплаты',
                    'type' => 'select',
                    'value' => '',
                );
            $ubrr_payment_declined =
                Array(
                    'GROUP' => $group,
                    'title' => 'Статус заказа после отклонения оплаты банком',
                    'type' => 'select',
                    'value' => '',
                );
            $ubrr_payment_canceled =
                Array(
                    'GROUP' => $group,
                    'title' => 'Статус заказа после отмены оплаты клиентом',
                    'type' => 'select',
                    'value' => '',
                );

            self::$optionList[$init_key] = $ubrr_payment_init;
            self::$optionList[$approved_key] = $ubrr_payment_approved;
            self::$optionList[$declined_key] = $ubrr_payment_declined;
            self::$optionList[$canceled_key] = $ubrr_payment_canceled;
        }
    }

    static function FillViewData()
    {
        global $view_data;
        $view_data = Array();

        $view_data['ubrr_wordpress_module'] = new Wordpress();

        //Разбиваем настройки на группы input'ов (UBRR_SETTINGS, UBRR_NOTIFICATIONS, etc)
        foreach (self::$optionList as $key => $inputData) {
            $group = 'INPUTS';
            if (isset($inputData['GROUP']) && !empty($inputData['GROUP']))
                $group = $inputData['GROUP'];

            //Получаем значения полей
            $inputData['value'] = get_option($key);

            if (!is_array($view_data[$group]))
                $view_data[$group] = Array();

            $view_data[$group][$key] = $inputData;
        }
    }

    function ShowSettingsPage()
    {
        self::FillViewData();
        include('views/admin_settings.php');
    }

    static function ActivatePlugin()
    {
        self::AddTable();
        $wp_ubrr = new Wordpress();
        $wp_ubrr->AddPaySystem();
    }



    public function RegisterWPOptions()
    {
        foreach (self::$optionList as $key => $optionData) {
            $defaultValue = $optionData['value'];
            if ($key == 'ubrr_shop_email') {
                $defaultValue = get_option('admin_email');
            }

            add_option($key, $defaultValue);
        }
    }

    static function AddTable()
    {
        //Создаем таблицу для заказов TWPG в БД
        global $wpdb;
        $table_name = $wpdb->prefix . "ubrr_twpg_orders";
        if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {
            $sql = "CREATE TABLE " . $table_name . " (
                  id INT NOT NULL AUTO_INCREMENT,
                  wp_id VARCHAR(255) NOT NULL,
                  twpg_id VARCHAR(255) NOT NULL,
                  session_id VARCHAR(255) NOT NULL,
                  ecom_session_id VARCHAR(255) NOT NULL,
                  PRIMARY KEY id (id)
                );";
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
        }
    }

    static function DropTable()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . "ubrr_twpg_orders";
        if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") == $table_name) {
            $sql = "DROP TABLE " . $table_name . ";";
            $wpdb->query($sql);
        }
    }

    function AddPaySystem()
    {
        //Проверяем какой плагин интернет магазина установлен
        $all_plugins = get_plugins();

        foreach ($all_plugins as $plugin) {
            $plugin_name = trim($plugin['Name']);

            if (isset(self::$known_shops_plugins[$plugin_name])) {
                $current_plugin_name = self::$known_shops_plugins[$plugin_name];
                //Проверяем наличие нужной папки, если ее нет - создаем
                $pay_system_absolute_path = $_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/' . $current_plugin_name['folder_name'];
                if (!file_exists($pay_system_absolute_path)) {
                    mkdir($pay_system_absolute_path);
                }
                //Создаем файл платежной системы
                $file_name = $current_plugin_name['file'];
                $payment_link_file = fopen($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/' . $current_plugin_name['folder_name'] . $file_name, 'w');
                $link_code = '<?php include_once("' . $_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/ubrr/shop_plugins/' . $file_name . '") ?>';
                fwrite($payment_link_file, $link_code);

                //Создаем лого, если WP-Shop
                if($plugin_name == 'WP-Shop'){
                copy($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/ubrr/views/images/wpshop_ubrr_logo.png',$_SERVER['DOCUMENT_ROOT'].'/wp-content/plugins/wp-shop-original/images/payments/wpshop_ubrr_logo.png');
                }
            }
        }
    }

    static function ManualAddPaySystem($plugin_name)
    {
        $plugin = self::$known_shops_plugins[$plugin_name];

        $payment_link_file = fopen($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/' . $plugin['folder_name'] . $plugin['file'], 'w');
        $link_code = '<?php include_once("' . $_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/ubrr/shop_plugins/' . $plugin['file'] . '") ?>';
        fwrite($payment_link_file, $link_code);
        if ($plugin_name == 'WooCommerce') {
            include_once('shop_plugins/class-wc-gateway-ubrr.php');
        }
        //Создаем лого, если WP-Shop
        if($plugin_name == 'WP-Shop'){
            copy($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/ubrr/views/images/wpshop_ubrr_logo.png',$_SERVER['DOCUMENT_ROOT'].'/wp-content/plugins/wp-shop-original/images/payments/wpshop_ubrr_logo.png');
        }
    }

    static function ManualDeletePaySystem($plugin_name)
    {
        $plugin = self::$known_shops_plugins[$plugin_name];
        unlink($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/' . $plugin['folder_name'] . $plugin['file']);
    }

    /**
     * При деактивации/удалении плагина убираем за собой
     */
    static function DeactivatePlugin()
    {
        self::DeletePaymentSystem();
        self::DropTable();
    }

    static function DeletePaymentSystem()
    {
        $wp_ubrr = new Wordpress();
        $wp_ubrr->PluginUninstall();
    }

    static function get_plugin_file($plugin_name)
    {
        require_once(ABSPATH . '/wp-admin/includes/plugin.php');
        $plugins = get_plugins();
        foreach ($plugins as $plugin_file => $plugin_info) {
            if ($plugin_info['Name'] == $plugin_name) return $plugin_file;
        }
        return null;
    }

    static function Init()
    {
        $directory = realpath(__DIR__) . '/shop_plugins_init';
        $initScripts = scandir($directory);

        unset($initScripts[0]);
        unset($initScripts[1]);
        foreach ($initScripts as $file) {
            $extension = end(explode(".", $file));
            if ($extension === 'php') {
                include_once($directory . '/' . $file);
            }
        }
    }
}