<?php

namespace UBRIR;

abstract class PaymentMailNotification implements InterfaceMailNotificationsConfig
{
    var $fromName = 'Платежный шлюз УБРиР';
    var $fromEmail = 'info@s.laa.net.ru';
    protected $viewData = Array();
    protected $paymentResultViewer;

    public function ConnectResultViewer(PaymentResultViewer $paymentResultViewer)
    {
        $this->paymentResultViewer = $paymentResultViewer;

        return $this;
    }

    public static function UTF8_Encode($CyrillicString)
    {
        return '=?utf-8?b?' . base64_encode($CyrillicString) . '?=';
    }

    public function GetOrderNumber()
    {
        if (!isset($this->viewData['v_order_number']))
            $this->viewData['v_order_number'] = $this->paymentResultViewer->GetOrderId();

        return $this->viewData['v_order_number'];
    }

    public function GetCardNumber()
    {
        if (!isset($this->viewData['v_card_number']))
            $this->viewData['v_card_number'] = $this->paymentResultViewer->GetCardNumber();

        return $this->viewData['v_card_number'];
    }

    public function GetCardHolderName()
    {
        if (!isset($this->viewData['v_card_holder_name']))
            $this->viewData['v_card_holder_name'] = $this->paymentResultViewer->GetCardHolderName();

        return $this->viewData['v_card_holder_name'];
    }

    public function GetOrderAmount()
    {
        if (!isset($this->viewData['v_order_amount']))
            $this->viewData['v_order_amount'] = floatval($this->paymentResultViewer->GetOrderAmount());

        return $this->viewData['v_order_amount'];
    }

    public function GetOrderDate()
    {
        if (!isset($this->viewData['v_order_date']))
            $this->viewData['v_order_date'] = new \DateTime($this->paymentResultViewer->GetPaymentTime());

        return $this->viewData['v_order_date'];
    }

    public function GetOperationType()
    {
        if (!isset($this->viewData['v_operation_type']))
            $this->viewData['v_operation_type'] = $this->paymentResultViewer->GetOperationType();

        return $this->viewData['v_operation_type'];
    }


    //TODO: вынести получения параметров письма за интерфейс, по аналогии с хранением TWPG информации и общими настройками ubrr.core

    public function GetTemplateViewData()
    {
        $this->GetLogoURL();

        if ($this->isNeedItSiteURL())
            $this->GetSiteURL();

        if ($this->isNeedItShopEmail())
            $this->GetShopEmail();

        if ($this->isNeedItOrderAmount())
            $this->GetOrderAmount();

        if ($this->isNeedItOrderDate())
            $this->GetOrderDate();

        if ($this->isNeedItOrderNumber())
            $this->GetOrderNumber();

        if ($this->isNeedItCardNumber())
            $this->GetCardNumber();

        if ($this->isNeedItCardHolderName())
            $this->GetCardHolderName();

        if ($this->isNeedItOperationType())
            $this->GetOperationType();

        return $this->viewData;
    }

    public function ChangeTemplateData($key, $value)
    {
        $this->viewData[$key] = $value;

        return $this;
    }

    public function SendPaymentNotification($receiverEmail, $receiverName = '')
    {
        $arTo = Array();
        if (!is_array($receiverEmail)) {
            if (!is_array($receiverName))
                $arTo = Array($receiverName => $receiverEmail);
        } else {
            $arTo = $receiverEmail;
        }
        //$to = implode(', ', array_values($arTo));
        $to = '';
        foreach ($arTo as $name => $email) {
            if (is_string($name) && strlen($name) > 0)
                $to .= self::UTF8_Encode($name) . ' <' . $email . '>, ';
            else
                $to .= '<' . $email . '>, ';
        }
        $to = substr($to, 0, strlen($to) - 2); //Удаляем последний [, ]

        // тема письма
        $subject = 'Уведомление о платеже';

        $curFileDir = dirname(realpath(__FILE__));
        // текст письма
        $mailTemplate = realpath($curFileDir . '/../views/ubrr.mail.admin_notification_simple.php');

        if ($mailTemplate) {
            global $view_data;
            $view_data = $this->GetTemplateViewData();

            ob_start();
            include_once $mailTemplate;
            $message = ob_get_contents();
            ob_end_clean();

            if ($this->isNotificationsEnabled()) {

                // Для отправки HTML-письма должен быть установлен заголовок Content-type
                $headers = 'MIME-Version: 1.0' . "\r\n";
                $headers .= 'Content-type: text/html; charset=UTF-8' . "\r\n";
                $headers .= 'From: ' . self::UTF8_Encode($this->fromName) . ' <' . $this->fromEmail . '>' . "\r\n";
                // Отправляем
                return mail($to, $subject, $message, $headers);
            } else {
                return false;
            }

        }
        return false;
    }
}