<?php

namespace UBRIR;

class ServerConnector
{
    const ServerURI_TEST = 'https://91.208.121.201:7443/Exec';
    const ServerURI_NORMAL = 'https://twpg.ubrr.ru:8443/Exec';

    public static function GetOrderStatus(GatewayInteractionPanel $gatewayPanel)
    {
        $shopId = $gatewayPanel->getShopId();
        $TWPG_OrderId = $gatewayPanel->getTWPG_OrderId();
        $TWPG_SessionId = $gatewayPanel->getTWPG_SessionId();

        $xml = '<?xml version = "1.0" encoding = "UTF-8"?>
                  <TKKPG>
                    <Request>
                      <Operation>GetOrderStatus</Operation>
                      <Language>RU</Language>
                      <Order>
                        <Merchant>' . $shopId . '</Merchant>
                        <OrderID>' . $TWPG_OrderId . '</OrderID>
                      </Order>
                      <SessionID>' . $TWPG_SessionId . '</SessionID>
                    </Request>
                  </TKKPG>';
        //return $this->xml_extract_status_result($this->send_xml($data), $status);

        return self::SendXML($xml, $gatewayPanel);
    }

    public static function GetJournal(GatewayInteractionPanel $gatewayPanel)
    {
        $shopId = $gatewayPanel->getShopId();

        $xml = '<?xml version = "1.0" encoding = "UTF-8"?>
                <TKKPG>
                  <Request>
                    <Operation>TransactionLog</Operation>
                    <Language>RU</Language>
                    <Merchant>' . $shopId . '</Merchant>
                  </Request>
                </TKKPG>';

        return self::SendXML($xml, $gatewayPanel);
    }

    public static function GetOrderInformation(GatewayInteractionPanel $gatewayPanel)
    {
        //Подтягиваем настройки соедения через коннектор к CMS
        $gatewayPanel->ServerConnectionConfig();

        $shopId = $gatewayPanel->getShopId();
        $TWPG_OrderId = $gatewayPanel->getTWPG_OrderId();
        $TWPG_SessionId = $gatewayPanel->getTWPG_SessionId();

        $xml = '<?xml version = "1.0" encoding = "UTF-8"?>
                  <TKKPG>
                    <Request>
                      <Operation>GetOrderInformation</Operation>
                      <Language>RU</Language>
                      <Order>
                        <Merchant>' . $shopId . '</Merchant>
                        <OrderID>' . $TWPG_OrderId . '</OrderID>
                      </Order>
                      <SessionID>' . $TWPG_SessionId . '</SessionID>
                      <ShowParams>true</ShowParams>
                      <ShowOperations>true</ShowOperations>
                      <ClassicView>true</ClassicView>
                    </Request>
                  </TKKPG>
          ';

        return self::SendXML($xml, $gatewayPanel);
    }

    public static function Reconcile(GatewayInteractionPanel $gatewayPanel)
    {
        $shopId = $gatewayPanel->getShopId();

        $xml = '<?xml version = "1.0" encoding = "UTF-8"?>
                <TKKPG>
                  <Request>
                    <Operation>Reconcile</Operation>
                    <Language>RU</Language>
                    <Merchant>' . $shopId . '</Merchant>
                  </Request>
                </TKKPG>';

        return self::SendXML($xml, $gatewayPanel);
    }

    public static function PrepareToPay(GatewayInteractionPanel $gatewayPanel)
    {
        $Amount = $gatewayPanel->getAmount();
        $Merchant = $gatewayPanel->getShopId();

        $ApproveURL = $gatewayPanel->getApproveURL();
        $CancelURL = $gatewayPanel->getApproveURL();
        $DeclineURL = $gatewayPanel->getApproveURL();

        $xml = '<?xml version = "1.0" encoding = "UTF-8"?>
                  <TKKPG>
                    <Request>
                      <Operation>CreateOrder</Operation>
                      <Language>RU</Language>
                      <Order>
                        <OrderType>Purchase</OrderType>
                        <Merchant>' . $Merchant . '</Merchant>
                        <Amount>' . $Amount . '</Amount>
                        <Currency>643</Currency>
                        <Description>Товары</Description>
                        <ApproveURL>' . $ApproveURL . '</ApproveURL>
                        <CancelURL>' . $CancelURL . '</CancelURL>
                        <DeclineURL>' . $DeclineURL . '</DeclineURL>
                      </Order>
                    </Request>
                  </TKKPG>';

        return self::SendXML($xml, $gatewayPanel);
    }

    private static function SendXML($xml, GatewayInteractionPanel $gatewayPanel)
    {
        $gatewayPanel->ServerConnectionConfig();

        $checkSecurityResponse = $gatewayPanel->CheckSecurityConfiguration();
        if (!$checkSecurityResponse->isSuccess()) {
            $gatewayPanel->LogErrors($checkSecurityResponse);
            return $checkSecurityResponse;
        }

        $caFile = $gatewayPanel->getCACertificateFile();
        $certFile = $gatewayPanel->getPublicCertificateFile();
        $keyFile = $gatewayPanel->getPrivateCertificateFile();
        $privateCertPass = $gatewayPanel->getPrivateCertificatePass();

        $gatewayPanel->Log($certFile);
        $gatewayPanel->Log($keyFile);
        $gatewayPanel->Log($privateCertPass);
        $gatewayPanel->Log($caFile);

        if ($gatewayPanel->getWorkMode()->IsNormal()) {
            $serverURI = self::ServerURI_NORMAL;
        } else {
            $serverURI = self::ServerURI_TEST;
        }

        $gatewayPanel->Log('=> Передаем данные серверу (URL = ' . $serverURI . ')');
        $gatewayPanel->Log($xml);

        $ch = curl_init($serverURI);    // initialize curl handle
        curl_setopt($ch, CURLOPT_POST, 1);

        if ($gatewayPanel->getWorkMode()->IsNormal()) {
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        } else {
            //DevMode
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        }

        curl_setopt($ch, CURLOPT_SSLCERT, $certFile);
        curl_setopt($ch, CURLOPT_CAINFO, $caFile);

        // this with CURLOPT_SSLKEYPASSWD
        curl_setopt($ch, CURLOPT_SSLKEY, $keyFile);
        curl_setopt($ch, CURLOPT_SSLCERTPASSWD, $privateCertPass);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);

        curl_setopt($ch, CURLOPT_MAXCONNECTS, 2);
        curl_setopt($ch, CURLOPT_FORBID_REUSE, 1); //Закрываем принудительно подключение

        curl_setopt($ch, CURLOPT_HTTPHEADER, Array("Connection: close"));
        /*
         * CURLOPT_HEADERFUNCTION	Callback-функция принимает два параметра. Первым параметром является дескриптор cURL,
         * вторым параметром является строка с записываемыми заголовками. Заголовки должны быть записаны с помощью
         * данной callback-функции. Должна возвратить количество записанных байт.
         */


        // curl_setopt($ch, CURLOPT_STDERR, $stdout);

        $xmlResponse = curl_exec($ch);

        $response = null;
        if ($xmlResponse === false) {
            $response = new DataContainer();
            $response->SetWorkMode($gatewayPanel->getWorkMode());
            $response->AddError('Не удалось связаться с платежным шлюзом');
            $response->AddError(curl_error($ch), 'CURL_ERROR');
            $response->AddError(curl_errno($ch), 'CURL_ERROR_NUMBER');

            $gatewayPanel->LogErrors($response);
        } else {
            $gatewayPanel->Log('<= Получили ответ');
            $gatewayPanel->Log($xmlResponse);
            $objXML = simplexml_load_string($xmlResponse);

            $arKeyValueXML = ServerConnector::XML2Array($xmlResponse);

            $response = new DataContainer();
            $response->SetWorkMode($gatewayPanel->getWorkMode());
            $response->SaveXMLData($xmlResponse, $objXML, $arKeyValueXML);

            //Анализируем ответ на наличие ошибок
            $response = self::AnalyzeServerResponse($response);
            if (!$response->isSuccess()) {
                $gatewayPanel->LogErrors($response);
            }
        }
        curl_close($ch);
        unset($ch);

        return $response;
    }

    private static function AnalyzeServerResponse(DataContainer $serverResponse)
    {
        $responseStatus = $serverResponse->GetResponseStatus();

        if ($responseStatus == TypeEnums::ResponseStatus_Success) {
            $serverResponse->setSuccess();
        } else {
            $serverResponse->AddError(TypeEnums::GetLabel_ResponseStatus($responseStatus), 'ResponseStatusError');
        }

        return $serverResponse;
    }

    private static function normalizeSimpleXML($obj, $lastKey, &$arKeyValue)
    {
        $data = $obj;
        if (is_object($data)) {
            $data = get_object_vars($data);
        }
        if (is_array($data)) {
            foreach ($data as $key => $value) {
                $lastKey = $key;

                if (is_object($value) || is_array($value)) {
                    self::normalizeSimpleXML($value, $lastKey, $arKeyValue);
                } else {
                    //Добавление в результирующий массив (копия здесь для того чтобы немного сократить рекурсию
                    if (isset($arKeyValue[$lastKey])) {
                        if (!is_array($arKeyValue[$lastKey])) {
                            $oldValue = $arKeyValue[$lastKey];
                            $arKeyValue[$lastKey] = Array($oldValue);
                        }
                        $arKeyValue[$lastKey][] = $value;
                    } else {
                        $arKeyValue[$lastKey] = $value;
                    }

                }
            }
        } else {
            //Сложно сказать когда это сработает теперь :)
            $arKeyValue[$lastKey] = $data;
        }
    }

    /**
     * Сплющиваем XML до одномерного массива. При дублировании названий нод - конвертируем результат в массив
     * @param $xmlString
     * @return array
     */
    public static function XML2Array($xmlString)
    {
        $arKeyValue = Array();
        $lastKey = '';

        self::normalizeSimpleXML(simplexml_load_string($xmlString), $lastKey, $arKeyValue);

        return $arKeyValue;
    }

    public static function XML2NestedArray($xml)
    {
        $result = Array();

        function normalizeSimpleXML($obj, &$result)
        {
            $data = $obj;
            if (is_object($data)) {
                $data = get_object_vars($data);
            }
            if (is_array($data)) {
                foreach ($data as $key => $value) {
                    $res = null;

                    normalizeSimpleXML($value, $res);
                    if (($key == '@attributes') && ($key)) {
                        $result = $res;
                    } else {
                        $result[strtolower($key)] = $res;
                    }
                }
            } else {
                $result = $data;
            }
        }

        normalizeSimpleXML(simplexml_load_string($xml), $result);

        return $result;
    }
}